from abc import ABC, abstractmethod
from typing import List, Optional, Union, Any

from aiohttp.client import ClientSession

from vkmini.types import methods
from vkmini.utils import AbstractLogger
from vkmini.exceptions import VkResponseException, VkErrorCaptcha


class ExecuteResult:
    """Содержит результаты выполнения метода `execute`"""
    response: Any
    errors: List[VkResponseException]


class BaseCaptchaHandler(ABC):
    """
    Обработчик ошибки #14 (Captcha needed)

    Для обработки этой ошибки в конструктор класса `VkApi` необходимо
    передать объект, реализующий метод `solve_captcha`
    """
    RETRY_COUNT = 5

    @abstractmethod
    async def solve_captcha(self, error: VkErrorCaptcha) -> Optional[str]:
        """Возвращает строку -- решение капчи (captcha_key)"""
        raise NotImplementedError


class VkApi:
    """
    Класс для выполнения запросов к VK API

    Пример использования:
    ```
        vk = VkApi(access_token)

        conversation_list = await vk.messages.getConversations()
    ```
    """
    URL: str
    rps_delay: int
    user_id: Union[int, None]
    version: str
    lang: str
    access_token: str
    logger: Union[AbstractLogger, None]
    retries: int
    excepts: bool

    account: methods.account
    ads: methods.ads
    adsweb: methods.adsweb
    appWidgets: methods.appWidgets
    apps: methods.apps
    auth: methods.auth
    board: methods.board
    database: methods.database
    docs: methods.docs
    downloadedGames: methods.downloadedGames
    fave: methods.fave
    friends: methods.friends
    gifts: methods.gifts
    groups: methods.groups
    likes: methods.likes
    market: methods.market
    messages: methods.messages
    newsfeed: methods.newsfeed
    notes: methods.notes
    notifications: methods.notifications
    orders: methods.orders
    pages: methods.pages
    photos: methods.photos
    podcasts: methods.podcasts
    polls: methods.polls
    prettyCards: methods.prettyCards
    search: methods.search
    secure: methods.secure
    stats: methods.stats
    status: methods.status
    storage: methods.storage
    stories: methods.stories
    streaming: methods.streaming
    users: methods.users
    utils: methods.utils
    video: methods.video
    wall: methods.wall
    widgets: methods.widgets

    def __init__(
            self,
            access_token: str,
            excepts: bool = False,
            version: str = '5.131',
            retries: int = 0,
            sync_mode: bool = None,
            logger: AbstractLogger = None,
            session: ClientSession = None,
            captcha_handler: Union[BaseCaptchaHandler, None] = None
    ):
        """
        `access_token` -- ключ доступа VK API
        (https://dev.vk.com/api/access-token/getting-started)

        `excepts` -- генерировать ли VkResponseException при ошибках VK API
        (иначе возвращает объект ошибки, полученный от VK)

        `version` -- версия VK API (https://dev.vk.com/reference/versions)

        `retries` -- количество повторных попыток при возникновении ошибки
        TooManyRequests

        `logger` -- любой объект, имеющий атрибуты info, debug и warning,
        по умолчанию None, то есть логирование не ведется

        `session` -- aiohttp.ClientSession, см. описание `vkmini.set_session`

        `captcha_handler` -- экземпляр `vkmini.api.BaseCaptchaHandler`,
        используемый для обработки ошибки #14 (Captcha needed)
        """

    async def __call__(self, method: str, **kwargs) -> Any:
        """
        Выполняет вызов метода VK API

        `method` -- название метода, например 'messages.send'
        """

    async def execute(self, code: str) -> ExecuteResult:
        """
        Выполняет код на языке VKScript

        Описание метода: https://dev.vk.com/method/execute

        Возвращает объект `vkmini.api.ExecuteResult`
        """

    async def get_vk_id(self) -> int: ...

    async def wait_request_delay(self) -> None: ...


class GroupVkApi(VkApi):
    """
    Класс для обращения к API с увеличенным количеством запросов в секунду
    """



import json

from enum import Enum
from typing import Any, List, Dict, Union, Literal


__all__ = (
    'Button',
    'FormattableButton',
    'Keyboard'
)


ButtonColors = Literal[
    'primary',  # Обычная
    'secondary',  # "Бледная"
    'positive',  # Зелёная
    'negative',  # Красная
]
ButtonTypes = Literal[
    'text',
    'vkpay',
    'location',
    'callback',
    'open_app',
    'open_link',
]


class Button:
    class Color(str, Enum):
        PRIMARY = 'primary'      # Синяя, обычная
        SECONDARY = 'secondary'  # "Бледная"
        POSITIVE = 'positive'    # Зелёная
        NEGATIVE = 'negative'    # Красная

    class Type(str, Enum):
        TEXT = 'text'
        VKPAY = 'vkpay'
        LOCATION = 'location'
        CALLBACK = 'callback'
        OPEN_APP = 'open_app'
        OPEN_LINK = 'open_link'

    payload: str
    color: str
    label: str
    type: str

    def __init__(
            self,
            label: str,
            payload: Union[str, dict],
            type: Type = Type.TEXT,
            color: Color = Color.PRIMARY
    ):
        self.payload = payload
        if isinstance(self.payload, dict):
            self.payload = json.dumps(self.payload, ensure_ascii=False)
        self.color = color
        self.label = label
        self.type = type

    def __str__(self) -> str:
        return Keyboard(self).jsonize()

    def __format_button__(self, data: Dict[str, Any]) -> 'Button':
        return self

    def copy(self) -> 'Button':
        return Button(self.label, self.payload, self.type, self.color)

    @property
    def obj(self):
        return {
            'action': {
                'type': self.type,
                'label': self.label,
                'payload': self.payload
            },
            'color': self.color
        }


class FormattableButton(Button):
    """
    Позволяет не создавать клавиатуру каждый раз, когда нужно поменять
    пару параметров в `payload` или `label` кнопки.

    Форматирование осуществляется вызовом метода `.format` у объектов
    `payload` и `label`, для переопределения способа форматирования
    можно переопределить метод `__format_button__` у этого класса или
    при создании кнопки передавать вместо `str` экземпляры своего класса

    Использование:
    ```
    keyboard = Keyboard([
        Button('Обычная кнопка', 'статичная нагрузочка'),
        FormattableButton('Label с {label_var}', 'Payload с {payload_var}')
    ])

    data = {
        'label_var': 'ярлычком',
        'payload_var': 'нагрузочкой'
    }
    formatted_kb = keyboard.format(data)  # вернёт клавиатуру с форматированными кнопками

    formatted_kb.jsonize()
    >>> ... {"label": "Label с ярлычком", "payload": "Payload с нагрузочкой"}
    ```
    """
    def __format_button__(self, data: Dict[str, Any]) -> 'Button':
        return Button(self.label.format(**data),
                      self.payload.format(**data),
                      self.type,
                      self.color)


class Keyboard:
    buttons: List[List[Button]]
    one_time: bool
    inline: bool

    def __init__(
            self,
            buttons: Union[Button, List[Button], List[List[Button]]] = None,
            inline: bool = True,
            one_time: bool = False
    ):
        self.buttons = []
        if buttons is not None:
            self.add_buttons(buttons)
        if inline and one_time:
            raise ValueError('inline и one_time -- взаимоисключающие параметры')
        self.inline = inline
        self.one_time = one_time

    def __str__(self) -> str:
        return self.jsonize()

    def add_new_button(self,
                       label: str,
                       payload: Union[str, dict],
                       type: Button.Type = Button.Type.TEXT,
                       color: Button.Color = Button.Color.PRIMARY):
        self.add_buttons(Button(label, payload, type, color))

    def add_buttons(self, buttons: Union[Button, List[Button]]):
        if isinstance(buttons, Button):
            buttons = [buttons]
        self.buttons.append(buttons)

    def jsonize(self) -> str:
        return json.dumps({
            'one_time': self.one_time,
            'inline': self.inline,
            'buttons': [[btn.obj for btn in line] for line in self.buttons]
        }, ensure_ascii=False)

    def copy(self) -> 'Keyboard':
        return [[btn.copy() for btn in line] for line in self.buttons]

    def format(self, data: Dict[str, Any]) -> 'Keyboard':
        formatted_keyboard = Keyboard()
        for btn_line in self.buttons:
            formatted_keyboard.add_buttons(
                [btn.__format_button__(data) for btn in btn_line]
            )
        return formatted_keyboard

    format.__doc__ = FormattableButton.__doc__



from typing import Coroutine, List, TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .api import VkApi


_API_METHOD_GROUPS = (
    'account',
    'ads',
    'apps',
    'audio',
    'board',
    'database',
    'docs',
    'fave',
    'friends',
    'gifts',
    'groups',
    'likes',
    'market',
    'messages',
    'newsfeed',
    'notes',
    'notifications',
    'pages',
    'photos',
    'places',
    'polls',
    'search',
    'secure',
    'store',
    'stats',
    'status',
    'storage',
    'users',
    'utils',
    'video',
    'podcasts',
    'leadforms',
    'prettycards',
    'stories',
    'appwidgets',
    'streaming',
    'orders',
    'wall',
    'widgets'
)


class MethodGroup:
    name: str

    @staticmethod
    def _get(vk: 'VkApi', name: str) -> Optional['MethodGroup']:
        if name in _API_METHOD_GROUPS:
            return MethodGroup(name, vk)
        return None

    @classmethod
    def _get_all(cls, vk: 'VkApi') -> List['MethodGroup']:
        return [cls(name, vk) for name in _API_METHOD_GROUPS.keys()]

    def __init__(self, name, vk) -> None:
        self.name = name
        self.vk = vk

    def __getattr__(self, method: str) -> Coroutine:
        def method_call(**kwargs):
            return self.vk(f'{self.name}.{method}', **kwargs)
        return method_call



class VkResponseException(Exception):
    """
    Описывает ошибку VK API

    `request_data` -- параметры запроса
    `error_code` -- код ошибки
    `error_msg` -- сообщение ошибки
    `error_data` -- объект из поля 'error' ответа VK API, описывающий ошибку
    """
    request_data: dict
    error_code: int
    error_msg: str
    error_data: dict

    def __new__(cls, error: dict, request_data: dict = {}):
        obj = Exception.__new__(_error_map.get(error['error_code'], cls))
        obj.__init__(error, request_data)
        return obj

    def __reduce_ex__(self, _):
        return self.__class__, (self.error_data, self.request_data)

    def __init__(self, error: dict, request_data: dict = {}):
        self.error_code = error['error_code']
        self.error_msg = error['error_msg']
        self.request_data = request_data
        self.error_data = error

    def __repr__(self) -> str:
        return f'{self.__class__.__name__}({self.error_code})'

    def __str__(self) -> str:
        info = f'Ошибка ВК #{self.error_code}: {self.error_msg}'
        if self.request_data:
            info += f'\nПараметры запроса: {self.request_data}'
        return info

    def get_short_str(self) -> str:
        return f'#{self.error_code}: {self.error_msg}'


class NetworkError(Exception):
    """Ошибка сети при выполнении запроса"""
    code: int

    def __init__(self, code: int):
        self.code = code


class TokenInvalid(VkResponseException):
    """
    Общая ошибка недействительного токена для групп, пользователей
    и приложений (коды 5, 27, 28)
    """


class VkErrorUnknown(VkResponseException):  # общая ошибка
    '''(код 1) Unknown error occurred'''

class VkErrorDisabled(VkResponseException):  # общая ошибка
    '''(код 2) Application is disabled. Enable your application or use test mode'''

class VkErrorMethod(VkResponseException):  # общая ошибка
    '''(код 3) Unknown method passed'''

class VkErrorSignature(VkResponseException):  # общая ошибка
    '''(код 4) Incorrect signature'''

class VkErrorAuth(TokenInvalid):  # общая ошибка
    '''(код 5) User authorization failed'''

class VkErrorTooMany(VkResponseException):  # общая ошибка
    '''(код 6) Too many requests per second'''

class VkErrorPermission(VkResponseException):  # общая ошибка
    '''(код 7) Permission to perform this action is denied'''

class VkErrorRequest(VkResponseException):  # общая ошибка
    '''(код 8) Invalid request'''

class VkErrorFlood(VkResponseException):  # общая ошибка
    '''(код 9) Flood control'''

class VkErrorServer(VkResponseException):  # общая ошибка
    '''(код 10) Internal server error'''

class VkErrorEnabledInTest(VkResponseException):  # общая ошибка
    '''(код 11) In test mode application should be disabled or user should be authorized'''

class VkErrorCompile(VkResponseException):
    '''(код 12) Unable to compile code'''

class VkErrorRuntime(VkResponseException):
    '''(код 13) Runtime error occurred during code invocation'''

class VkErrorCaptcha(VkResponseException):  # общая ошибка
    '''(код 14) Captcha needed'''

class VkErrorAccess(VkResponseException):  # общая ошибка
    '''(код 15) Access denied'''

class VkErrorAuthHttps(VkResponseException):  # общая ошибка
    '''(код 16) HTTP authorization failed'''

class VkErrorAuthValidation(VkResponseException):  # общая ошибка
    '''(код 17) Validation required'''

class VkErrorUserDeleted(VkResponseException):  # общая ошибка
    '''(код 18) User was deleted or banned'''

class VkErrorBlocked(VkResponseException):
    '''(код 19) Content blocked'''

class VkErrorMethodPermission(VkResponseException):  # общая ошибка
    '''(код 20) Permission to perform this action is denied for non-standalone applications'''

class VkErrorMethodAds(VkResponseException):  # общая ошибка
    '''(код 21) Permission to perform this action is allowed only for standalone and OpenAPI applications'''

class VkErrorUpload(VkResponseException):
    '''(код 22) Upload error'''

class VkErrorMethodDisabled(VkResponseException):  # общая ошибка
    '''(код 23) This method was disabled'''

class VkErrorNeedConfirmation(VkResponseException):  # общая ошибка
    '''(код 24) Confirmation required'''

class VkErrorNeedTokenConfirmation(VkResponseException):  # общая ошибка
    '''(код 25) Token confirmation required'''

class VkErrorGroupAuth(TokenInvalid):  # общая ошибка
    '''(код 27) Group authorization failed'''

class VkErrorAppAuth(TokenInvalid):  # общая ошибка
    '''(код 28) Application authorization failed'''

class VkErrorRateLimit(VkResponseException):  # общая ошибка
    '''(код 29) Rate limit reached'''

class VkErrorPrivateProfile(VkResponseException):  # общая ошибка
    '''(код 30) This profile is private'''

class VkErrorNotImplementedYet(VkResponseException):  # общая ошибка
    '''(код 33) Not implemented yet'''

class VkErrorClientUpdateNeeded(VkResponseException):
    '''(код 35) Client update needed'''

class VkErrorParam(VkResponseException):  # общая ошибка
    '''(код 100) One of the parameters specified was missing or invalid'''

class VkErrorParamApiId(VkResponseException):  # общая ошибка
    '''(код 101) Invalid application API ID'''

class VkErrorLimits(VkResponseException):
    '''(код 103) Out of limits'''

class VkErrorNotFound(VkResponseException):
    '''(код 104) Not found'''

class VkErrorSaveFile(VkResponseException):
    '''(код 105) Couldn't save file'''

class VkErrorActionFailed(VkResponseException):
    '''(код 106) Unable to process action'''

class VkErrorParamUserId(VkResponseException):  # общая ошибка
    '''(код 113) Invalid user id'''

class VkErrorParamAlbumId(VkResponseException):
    '''(код 114) Invalid album id'''

class VkErrorParamServer(VkResponseException):
    '''(код 118) Invalid server'''

class VkErrorParamTitle(VkResponseException):
    '''(код 119) Invalid title'''

class VkErrorParamHash(VkResponseException):
    '''(код 121) Invalid hash'''

class VkErrorParamPhotos(VkResponseException):
    '''(код 122) Invalid photos'''

class VkErrorParamGroupId(VkResponseException):
    '''(код 125) Invalid group id'''

class VkErrorParamPhoto(VkResponseException):
    '''(код 129) Invalid photo'''

class VkErrorParamPageId(VkResponseException):
    '''(код 140) Page not found'''

class VkErrorAccessPage(VkResponseException):
    '''(код 141) Access to page denied'''

class VkErrorMobileNotActivated(VkResponseException):
    '''(код 146) The mobile number of the user is unknown'''

class VkErrorInsufficientFunds(VkResponseException):
    '''(код 147) Application has insufficient funds'''

class VkErrorAccessMenu(VkResponseException):
    '''(код 148) Access to the menu of the user denied'''

class VkErrorParamTimestamp(VkResponseException):  # общая ошибка
    '''(код 150) Invalid timestamp'''

class VkErrorFriendsListId(VkResponseException):
    '''(код 171) Invalid list id'''

class VkErrorFriendsListLimit(VkResponseException):
    '''(код 173) Reached the maximum number of lists'''

class VkErrorFriendsAddYourself(VkResponseException):
    '''(код 174) Cannot add user himself as friend'''

class VkErrorFriendsAddInEnemy(VkResponseException):
    '''(код 175) Cannot add this user to friends as they have put you on their blacklist'''

class VkErrorFriendsAddEnemy(VkResponseException):
    '''(код 176) Cannot add this user to friends as you put him on blacklist'''

class VkErrorFriendsAddNotFound(VkResponseException):
    '''(код 177) Cannot add this user to friends as user not found'''

class VkErrorParamNoteId(VkResponseException):
    '''(код 180) Note not found'''

class VkErrorAccessNote(VkResponseException):
    '''(код 181) Access to note denied'''

class VkErrorAccessNoteComment(VkResponseException):
    '''(код 182) You can't comment this note'''

class VkErrorAccessComment(VkResponseException):
    '''(код 183) Access to comment denied'''

class VkErrorAccessAlbum(VkResponseException):  # общая ошибка
    '''(код 200) Access denied'''

class VkErrorAccessAudio(VkResponseException):  # общая ошибка
    '''(код 201) Access denied'''

class VkErrorAccessGroup(VkResponseException):  # общая ошибка
    '''(код 203) Access to group denied'''

class VkErrorAccessVideo(VkResponseException):
    '''(код 204) Access denied'''

class VkErrorAccessMarket(VkResponseException):
    '''(код 205) Access denied'''

class VkErrorWallAccessPost(VkResponseException):
    '''(код 210) Access to wall's post denied'''

class VkErrorWallAccessComment(VkResponseException):
    '''(код 211) Access to wall's comment denied'''

class VkErrorWallAccessReplies(VkResponseException):
    '''(код 212) Access to post comments denied'''

class VkErrorWallAccessAddReply(VkResponseException):
    '''(код 213) Access to status replies denied'''

class VkErrorWallAddPost(VkResponseException):
    '''(код 214) Access to adding post denied'''

class VkErrorWallAdsPublished(VkResponseException):
    '''(код 219) Advertisement post was recently added'''

class VkErrorWallTooManyRecipients(VkResponseException):
    '''(код 220) Too many recipients'''

class VkErrorStatusNoAudio(VkResponseException):
    '''(код 221) User disabled track name broadcast'''

class VkErrorWallLinksForbidden(VkResponseException):
    '''(код 222) Hyperlinks are forbidden'''

class VkErrorWallReplyOwnerFlood(VkResponseException):
    '''(код 223) Too many replies'''

class VkErrorWallAdsPostLimitReached(VkResponseException):
    '''(код 224) Too many ads posts'''

class VkErrorPollsAccess(VkResponseException):
    '''(код 250) Access to poll denied'''

class VkErrorPollsPollId(VkResponseException):
    '''(код 251) Invalid poll id'''

class VkErrorPollsAnswerId(VkResponseException):
    '''(код 252) Invalid answer id'''

class VkErrorPollsAccessWithoutVote(VkResponseException):
    '''(код 253) Access denied, please vote first'''

class VkErrorAccessGroups(VkResponseException):
    '''(код 260) Access to the groups list is denied due to the user's privacy settings'''

class VkErrorAlbumFull(VkResponseException):  # общая ошибка
    '''(код 300) This album is full'''

class VkErrorAlbumsLimit(VkResponseException):
    '''(код 302) Albums number limit is reached'''

class VkErrorVotesPermission(VkResponseException):  # общая ошибка
    '''(код 500) Permission denied. You must enable votes processing in application settings'''

class VkErrorVotes(VkResponseException):
    '''(код 503) Not enough votes'''

class VkErrorAdsPermission(VkResponseException):  # общая ошибка
    '''(код 600) Permission denied. You have no access to operations specified with given object(s)'''

class VkErrorWeightedFlood(VkResponseException):
    '''(код 601) Permission denied. You have requested too many actions this day. Try later.'''

class VkErrorAdsPartialSuccess(VkResponseException):
    '''(код 602) Some part of the request has not been completed'''

class VkErrorAdsSpecific(VkResponseException):  # общая ошибка
    '''(код 603) Some ads error occured'''

class VkErrorAdsObjectDeleted(VkResponseException):
    '''(код 629) Object deleted'''

class VkErrorGroupChangeCreator(VkResponseException):
    '''(код 700) Cannot edit creator role'''

class VkErrorGroupNotInClub(VkResponseException):
    '''(код 701) User should be in club'''

class VkErrorGroupTooManyOfficers(VkResponseException):
    '''(код 702) Too many officers in club'''

class VkErrorGroupNeed2fa(VkResponseException):
    '''(код 703) You need to enable 2FA for this action'''

class VkErrorGroupHostNeed2fa(VkResponseException):
    '''(код 704) User needs to enable 2FA for this action'''

class VkErrorGroupTooManyAddresses(VkResponseException):
    '''(код 706) Too many addresses in club'''

class VkErrorGroupAppIsNotInstalledInCommunity(VkResponseException):
    '''(код 711) Application is not installed in community'''

class VkErrorGroupInviteLinksNotValid(VkResponseException):
    '''(код 714) Invite link is invalid - expired, deleted or not exists'''

class VkErrorVideoAlreadyAdded(VkResponseException):
    '''(код 800) This video is already added'''

class VkErrorVideoCommentsClosed(VkResponseException):
    '''(код 801) Comments for this video are closed'''

class VkErrorMessagesUserBlocked(VkResponseException):
    '''(код 900) Can't send messages for users from blacklist'''

class VkErrorMessagesDenySend(VkResponseException):
    '''(код 901) Can't send messages for users without permission'''

class VkErrorMessagesPrivacy(VkResponseException):
    '''(код 902) Can't send messages to this user due to their privacy settings'''

class VkErrorMessagesTooOldPts(VkResponseException):
    '''(код 907) Value of ts or pts is too old'''

class VkErrorMessagesTooNewPts(VkResponseException):
    '''(код 908) Value of ts or pts is too new'''

class VkErrorMessagesEditExpired(VkResponseException):
    '''(код 909) Can't edit this message, because it's too old'''

class VkErrorMessagesTooBig(VkResponseException):
    '''(код 910) Can't sent this message, because it's too big'''

class VkErrorMessagesKeyboardInvalid(VkResponseException):
    '''(код 911) Keyboard format is invalid'''

class VkErrorMessagesChatBotFeature(VkResponseException):
    '''(код 912) This is a chat bot feature, change this status in settings'''

class VkErrorMessagesTooLongForwards(VkResponseException):
    '''(код 913) Too many forwarded messages'''

class VkErrorMessagesTooLongMessage(VkResponseException):
    '''(код 914) Message is too long'''

class VkErrorMessagesChatUserNoAccess(VkResponseException):
    '''(код 917) You don't have access to this chat'''

class VkErrorMessagesCantSeeInviteLink(VkResponseException):
    '''(код 919) You can't see invite link for this chat'''

class VkErrorMessagesEditKindDisallowed(VkResponseException):
    '''(код 920) Can't edit this kind of message'''

class VkErrorMessagesCantFwd(VkResponseException):
    '''(код 921) Can't forward these messages'''

class VkErrorMessagesCantDeleteForAll(VkResponseException):
    '''(код 924) Can't delete this message for everybody'''

class VkErrorMessagesChatNotAdmin(VkResponseException):
    '''(код 925) You are not admin of this chat'''

class VkErrorMessagesChatNotExist(VkResponseException):
    '''(код 927) Chat does not exist'''

class VkErrorMessagesCantChangeInviteLink(VkResponseException):
    '''(код 931) You can't change invite link for this chat'''

class VkErrorMessagesGroupPeerAccess(VkResponseException):
    '''(код 932) Your community can't interact with this peer'''

class VkErrorMessagesChatUserNotInChat(VkResponseException):
    '''(код 935) User not found in chat'''

class VkErrorMessagesContactNotFound(VkResponseException):
    '''(код 936) Contact not found'''

class VkErrorMessagesMessageRequestAlreadySent(VkResponseException):
    '''(код 939) Message request already sent'''

class VkErrorMessagesTooManyPosts(VkResponseException):
    '''(код 940) Too many posts in messages'''

class VkErrorMessagesCantPinOneTimeStory(VkResponseException):
    '''(код 942) Cannot pin one-time story'''

class VkErrorMessagesIntentCantUse(VkResponseException):
    '''(код 943) Cannot use this intent'''

class VkErrorMessagesIntentLimitOverflow(VkResponseException):
    '''(код 944) Limits overflow for this intent'''

class VkErrorMessagesChatDisabled(VkResponseException):
    '''(код 945) Chat was disabled'''

class VkErrorMessagesChatUnsupported(VkResponseException):
    '''(код 946) Chat not supported'''

class VkErrorMessagesMemberAccessToGroupDenied(VkResponseException):
    '''(код 947) Can't add user to chat, because user has no access to group'''

class VkErrorMessagesCantEditPinnedYet(VkResponseException):
    '''(код 949) Can't edit pinned message yet'''

class VkErrorMessagesPeerBlockedReasonByTime(VkResponseException):
    '''(код 950) Can't send message, reply timed out'''

class VkErrorParamPhone(VkResponseException):
    '''(код 1000) Invalid phone number'''

class VkErrorPhoneAlreadyUsed(VkResponseException):
    '''(код 1004) This phone number is used by another user'''

class VkErrorAuthFloodError(VkResponseException):
    '''(код 1105) Too many auth attempts, try again later'''

class VkErrorAuthDelay(VkResponseException):
    '''(код 1112) Processing.. Try later'''

class VkErrorParamDocId(VkResponseException):
    '''(код 1150) Invalid document id'''

class VkErrorParamDocDeleteAccess(VkResponseException):
    '''(код 1151) Access to document deleting is denied'''

class VkErrorParamDocTitle(VkResponseException):
    '''(код 1152) Invalid document title'''

class VkErrorParamDocAccess(VkResponseException):
    '''(код 1153) Access to document is denied'''

class VkErrorPhotoChanged(VkResponseException):
    '''(код 1160) Original photo was changed'''

class VkErrorTooManyLists(VkResponseException):
    '''(код 1170) Too many feed lists'''

class VkErrorAppsAlreadyUnlocked(VkResponseException):
    '''(код 1251) This achievement is already unlocked'''

class VkErrorAppsSubscriptionNotFound(VkResponseException):
    '''(код 1256) Subscription not found'''

class VkErrorAppsSubscriptionInvalidStatus(VkResponseException):
    '''(код 1257) Subscription is in invalid status'''

class VkErrorInvalidAddress(VkResponseException):
    '''(код 1260) Invalid screen name'''

class VkErrorCommunitiesCatalogDisabled(VkResponseException):
    '''(код 1310) Catalog is not available for this user'''

class VkErrorCommunitiesCategoriesDisabled(VkResponseException):
    '''(код 1311) Catalog categories are not available for this user'''

class VkErrorMarketRestoreTooLate(VkResponseException):
    '''(код 1400) Too late for restore'''

class VkErrorMarketCommentsClosed(VkResponseException):
    '''(код 1401) Comments for this market are closed'''

class VkErrorMarketAlbumNotFound(VkResponseException):
    '''(код 1402) Album not found'''

class VkErrorMarketItemNotFound(VkResponseException):
    '''(код 1403) Item not found'''

class VkErrorMarketItemAlreadyAdded(VkResponseException):
    '''(код 1404) Item already added to album'''

class VkErrorMarketTooManyItems(VkResponseException):
    '''(код 1405) Too many items'''

class VkErrorMarketTooManyItemsInAlbum(VkResponseException):
    '''(код 1406) Too many items in album'''

class VkErrorMarketTooManyAlbums(VkResponseException):
    '''(код 1407) Too many albums'''

class VkErrorMarketItemHasBadLinks(VkResponseException):
    '''(код 1408) Item has bad links in description'''

class VkErrorMarketShopNotEnabled(VkResponseException):
    '''(код 1409) Shop not enabled'''

class VkErrorMarketVariantNotFound(VkResponseException):
    '''(код 1416) Variant not found'''

class VkErrorMarketPropertyNotFound(VkResponseException):
    '''(код 1417) Property not found'''

class VkErrorMarketGroupingMustContainMoreThanOneItem(VkResponseException):
    '''(код 1425) Grouping must have two or more items'''

class VkErrorMarketGroupingItemsMustHaveDistinctProperties(VkResponseException):
    '''(код 1426) Item must have distinct properties'''

class VkErrorMarketOrdersNoCartItems(VkResponseException):
    '''(код 1427) Cart is empty'''

class VkErrorMarketInvalidDimensions(VkResponseException):
    '''(код 1429) Specify width, length, height and weight all together'''

class VkErrorMarketCantChangeVkpayStatus(VkResponseException):
    '''(код 1430) VK Pay status can not be changed'''

class VkErrorMarketShopAlreadyEnabled(VkResponseException):
    '''(код 1431) Market was already enabled in this group'''

class VkErrorMarketShopAlreadyDisabled(VkResponseException):
    '''(код 1432) Market was already disabled in this group'''

class VkErrorMarketPhotosCropInvalidFormat(VkResponseException):
    '''(код 1433) Invalid image crop format'''

class VkErrorMarketPhotosCropOverflow(VkResponseException):
    '''(код 1434) Crop bottom right corner is outside of the image'''

class VkErrorMarketPhotosCropSizeTooLow(VkResponseException):
    '''(код 1435) Crop size is less than the minimum'''

class VkErrorStoryExpired(VkResponseException):
    '''(код 1600) Story has already expired'''

class VkErrorStoryIncorrectReplyPrivacy(VkResponseException):
    '''(код 1602) Incorrect reply privacy'''

class VkErrorPrettyCardsCardNotFound(VkResponseException):
    '''(код 1900) Card not found'''

class VkErrorPrettyCardsTooManyCards(VkResponseException):
    '''(код 1901) Too many cards'''

class VkErrorPrettyCardsCardIsConnectedToPost(VkResponseException):
    '''(код 1902) Card is connected to post'''

class VkErrorCallbackApiServersLimit(VkResponseException):
    '''(код 2000) Servers number limit is reached'''

class VkErrorWallCheckLinkCantDetermineSource(VkResponseException):
    '''(код 3102) Specified link is incorrect (can't find source)'''

class VkErrorRecaptcha(VkResponseException):  # общая ошибка
    '''(код 3300) Recaptcha needed'''

class VkErrorPhoneValidationNeed(VkResponseException):  # общая ошибка
    '''(код 3301) Phone validation needed'''

class VkErrorPasswordValidationNeed(VkResponseException):  # общая ошибка
    '''(код 3302) Password validation needed'''

class VkErrorOtpValidationNeed(VkResponseException):  # общая ошибка
    '''(код 3303) Otp app validation needed'''

class VkErrorEmailConfirmationNeed(VkResponseException):  # общая ошибка
    '''(код 3304) Email confirmation needed'''

class VkErrorAssertVotes(VkResponseException):  # общая ошибка
    '''(код 3305) Assert votes'''

class VkErrorTokenExtensionRequired(VkResponseException):  # общая ошибка
    '''(код 3609) Token extension required'''

class VkErrorUserDeactivated(VkResponseException):  # общая ошибка
    '''(код 3610) User is deactivated'''

class VkErrorUserServiceDeactivated(VkResponseException):  # общая ошибка
    '''(код 3611) Service is deactivated for user'''

class VkErrorFaveAliexpressTag(VkResponseException):
    '''(код 3800) Can't set AliExpress tag to this type of object'''


_error_map = {
    1: VkErrorUnknown,
    2: VkErrorDisabled,
    3: VkErrorMethod,
    4: VkErrorSignature,
    5: VkErrorAuth,
    6: VkErrorTooMany,
    7: VkErrorPermission,
    8: VkErrorRequest,
    9: VkErrorFlood,
    10: VkErrorServer,
    11: VkErrorEnabledInTest,
    12: VkErrorCompile,
    13: VkErrorRuntime,
    14: VkErrorCaptcha,
    15: VkErrorAccess,
    16: VkErrorAuthHttps,
    17: VkErrorAuthValidation,
    18: VkErrorUserDeleted,
    19: VkErrorBlocked,
    20: VkErrorMethodPermission,
    21: VkErrorMethodAds,
    22: VkErrorUpload,
    23: VkErrorMethodDisabled,
    24: VkErrorNeedConfirmation,
    25: VkErrorNeedTokenConfirmation,
    27: VkErrorGroupAuth,
    28: VkErrorAppAuth,
    29: VkErrorRateLimit,
    30: VkErrorPrivateProfile,
    33: VkErrorNotImplementedYet,
    35: VkErrorClientUpdateNeeded,
    100: VkErrorParam,
    101: VkErrorParamApiId,
    103: VkErrorLimits,
    104: VkErrorNotFound,
    105: VkErrorSaveFile,
    106: VkErrorActionFailed,
    113: VkErrorParamUserId,
    114: VkErrorParamAlbumId,
    118: VkErrorParamServer,
    119: VkErrorParamTitle,
    121: VkErrorParamHash,
    122: VkErrorParamPhotos,
    125: VkErrorParamGroupId,
    129: VkErrorParamPhoto,
    140: VkErrorParamPageId,
    141: VkErrorAccessPage,
    146: VkErrorMobileNotActivated,
    147: VkErrorInsufficientFunds,
    148: VkErrorAccessMenu,
    150: VkErrorParamTimestamp,
    171: VkErrorFriendsListId,
    173: VkErrorFriendsListLimit,
    174: VkErrorFriendsAddYourself,
    175: VkErrorFriendsAddInEnemy,
    176: VkErrorFriendsAddEnemy,
    177: VkErrorFriendsAddNotFound,
    180: VkErrorParamNoteId,
    181: VkErrorAccessNote,
    182: VkErrorAccessNoteComment,
    183: VkErrorAccessComment,
    200: VkErrorAccessAlbum,
    201: VkErrorAccessAudio,
    203: VkErrorAccessGroup,
    204: VkErrorAccessVideo,
    205: VkErrorAccessMarket,
    210: VkErrorWallAccessPost,
    211: VkErrorWallAccessComment,
    212: VkErrorWallAccessReplies,
    213: VkErrorWallAccessAddReply,
    214: VkErrorWallAddPost,
    219: VkErrorWallAdsPublished,
    220: VkErrorWallTooManyRecipients,
    221: VkErrorStatusNoAudio,
    222: VkErrorWallLinksForbidden,
    223: VkErrorWallReplyOwnerFlood,
    224: VkErrorWallAdsPostLimitReached,
    250: VkErrorPollsAccess,
    251: VkErrorPollsPollId,
    252: VkErrorPollsAnswerId,
    253: VkErrorPollsAccessWithoutVote,
    260: VkErrorAccessGroups,
    300: VkErrorAlbumFull,
    302: VkErrorAlbumsLimit,
    500: VkErrorVotesPermission,
    503: VkErrorVotes,
    600: VkErrorAdsPermission,
    601: VkErrorWeightedFlood,
    602: VkErrorAdsPartialSuccess,
    603: VkErrorAdsSpecific,
    629: VkErrorAdsObjectDeleted,
    700: VkErrorGroupChangeCreator,
    701: VkErrorGroupNotInClub,
    702: VkErrorGroupTooManyOfficers,
    703: VkErrorGroupNeed2fa,
    704: VkErrorGroupHostNeed2fa,
    706: VkErrorGroupTooManyAddresses,
    711: VkErrorGroupAppIsNotInstalledInCommunity,
    714: VkErrorGroupInviteLinksNotValid,
    800: VkErrorVideoAlreadyAdded,
    801: VkErrorVideoCommentsClosed,
    900: VkErrorMessagesUserBlocked,
    901: VkErrorMessagesDenySend,
    902: VkErrorMessagesPrivacy,
    907: VkErrorMessagesTooOldPts,
    908: VkErrorMessagesTooNewPts,
    909: VkErrorMessagesEditExpired,
    910: VkErrorMessagesTooBig,
    911: VkErrorMessagesKeyboardInvalid,
    912: VkErrorMessagesChatBotFeature,
    913: VkErrorMessagesTooLongForwards,
    914: VkErrorMessagesTooLongMessage,
    917: VkErrorMessagesChatUserNoAccess,
    919: VkErrorMessagesCantSeeInviteLink,
    920: VkErrorMessagesEditKindDisallowed,
    921: VkErrorMessagesCantFwd,
    924: VkErrorMessagesCantDeleteForAll,
    925: VkErrorMessagesChatNotAdmin,
    927: VkErrorMessagesChatNotExist,
    931: VkErrorMessagesCantChangeInviteLink,
    932: VkErrorMessagesGroupPeerAccess,
    935: VkErrorMessagesChatUserNotInChat,
    936: VkErrorMessagesContactNotFound,
    939: VkErrorMessagesMessageRequestAlreadySent,
    940: VkErrorMessagesTooManyPosts,
    942: VkErrorMessagesCantPinOneTimeStory,
    943: VkErrorMessagesIntentCantUse,
    944: VkErrorMessagesIntentLimitOverflow,
    945: VkErrorMessagesChatDisabled,
    946: VkErrorMessagesChatUnsupported,
    947: VkErrorMessagesMemberAccessToGroupDenied,
    949: VkErrorMessagesCantEditPinnedYet,
    950: VkErrorMessagesPeerBlockedReasonByTime,
    1000: VkErrorParamPhone,
    1004: VkErrorPhoneAlreadyUsed,
    1105: VkErrorAuthFloodError,
    1112: VkErrorAuthDelay,
    1150: VkErrorParamDocId,
    1151: VkErrorParamDocDeleteAccess,
    1152: VkErrorParamDocTitle,
    1153: VkErrorParamDocAccess,
    1160: VkErrorPhotoChanged,
    1170: VkErrorTooManyLists,
    1251: VkErrorAppsAlreadyUnlocked,
    1256: VkErrorAppsSubscriptionNotFound,
    1257: VkErrorAppsSubscriptionInvalidStatus,
    1260: VkErrorInvalidAddress,
    1310: VkErrorCommunitiesCatalogDisabled,
    1311: VkErrorCommunitiesCategoriesDisabled,
    1400: VkErrorMarketRestoreTooLate,
    1401: VkErrorMarketCommentsClosed,
    1402: VkErrorMarketAlbumNotFound,
    1403: VkErrorMarketItemNotFound,
    1404: VkErrorMarketItemAlreadyAdded,
    1405: VkErrorMarketTooManyItems,
    1406: VkErrorMarketTooManyItemsInAlbum,
    1407: VkErrorMarketTooManyAlbums,
    1408: VkErrorMarketItemHasBadLinks,
    1409: VkErrorMarketShopNotEnabled,
    1416: VkErrorMarketVariantNotFound,
    1417: VkErrorMarketPropertyNotFound,
    1425: VkErrorMarketGroupingMustContainMoreThanOneItem,
    1426: VkErrorMarketGroupingItemsMustHaveDistinctProperties,
    1427: VkErrorMarketOrdersNoCartItems,
    1429: VkErrorMarketInvalidDimensions,
    1430: VkErrorMarketCantChangeVkpayStatus,
    1431: VkErrorMarketShopAlreadyEnabled,
    1432: VkErrorMarketShopAlreadyDisabled,
    1433: VkErrorMarketPhotosCropInvalidFormat,
    1434: VkErrorMarketPhotosCropOverflow,
    1435: VkErrorMarketPhotosCropSizeTooLow,
    1600: VkErrorStoryExpired,
    1602: VkErrorStoryIncorrectReplyPrivacy,
    1900: VkErrorPrettyCardsCardNotFound,
    1901: VkErrorPrettyCardsTooManyCards,
    1902: VkErrorPrettyCardsCardIsConnectedToPost,
    2000: VkErrorCallbackApiServersLimit,
    3102: VkErrorWallCheckLinkCantDetermineSource,
    3300: VkErrorRecaptcha,
    3301: VkErrorPhoneValidationNeed,
    3302: VkErrorPasswordValidationNeed,
    3303: VkErrorOtpValidationNeed,
    3304: VkErrorEmailConfirmationNeed,
    3305: VkErrorAssertVotes,
    3609: VkErrorTokenExtensionRequired,
    3610: VkErrorUserDeactivated,
    3611: VkErrorUserServiceDeactivated,
    3800: VkErrorFaveAliexpressTag,
}

# для совместимости со старыми версиями
RateLimit = VkErrorRateLimit
FloodControl = VkErrorFlood
CaptchaNeeded = VkErrorCaptcha
TooManyRequests = VkErrorTooMany



from typing import AsyncGenerator, List, Any, Union
from warnings import warn

from aiohttp.client import ClientSession

from vkmini.api import VkApi
from vkmini.utils import AbstractLogger
from vkmini.request import longpoll_get, _check_longpoll_session
from vkmini.exceptions import TokenInvalid


class LP:
    """
    Реализует получение событий через User Long Poll API

    Официальная документация:
    https://dev.vk.com/api/user-long-poll/getting-started

    Неофициальная документация:
    https://github.com/danyadev/longpoll-doc
    """
    ts: int
    key: str
    server: str

    mode: int
    wait: int

    _vk: VkApi
    _session: Union[ClientSession, None]
    __session_owner: bool = False

    def __init__(
            self,
            vk: VkApi,
            wait: int = 25,
            mode: int = 2,
            logger: AbstractLogger = None,
            session: ClientSession = None
        ):
        """
        `wait` -- максимальное время ожидания события, в секундах
        (https://dev.vk.com/api/user-long-poll/getting-started#Подключение)

        `mode` -- дополнительные опции ответа

        `logger` -- любой объект, имеющий атрибуты info, debug и warning,
        по умолчанию None, то есть логирование не ведется

        `session` -- экземпляр aiohttp.ClientSession, который будет
        использоваться при выполнении запросов к LongPoll серверу

        Приоритет объектов сессий:
        1. Сессия, переданная в параметре session
        2. Сессия переданного экземпляра VkApi
        3. Общая сессия (см vkmini.set_session)
        """
        self._init(vk, wait, mode, logger, session)

    def _init(self, vk, wait, mode, logger, session) -> None:
        self._vk = vk
        self.wait = wait
        self.mode = mode
        self.logger = logger or vk.logger
        self._session = session or vk._session

    async def check(self):
        """
        Выполняет однократный запрос к API и возвращает
        список событий из поля 'updates'
        (https://dev.vk.com/api/user-long-poll/getting-started#Формат%20ответа)
        """
        _check_longpoll_session(self._session)
        await self._set_longpoll_data(True)
        self.check = self._check
        return await self.check()

    async def _check(self) -> List[List[Any]]:
        data = await longpoll_get(
            f"https://{self.server}?act=a_check&key={self.key}"
            f"&ts={self.ts}&wait={self.wait}&mode={self.mode}&version=10",
            self._session
        )

        if 'failed' in data:
            if data['failed'] == 1:
                self.ts = data['ts']
            elif data['failed'] == 2:
                await self._set_longpoll_data(False)
            else:
                await self._set_longpoll_data(True)
            return []
        else:
            self.ts = data['ts']
            return data['updates']

    async def _set_longpoll_data(self, new_ts: bool) -> None:
        data = await self._vk.messages.getLongPollServer(
            need_pts=bool(self.mode & 32 == 32)
        )
        if not self._vk.excepts:
            if data.get('error', {}).get('error_code') == 5:
                raise TokenInvalid(data['error'])
        self.server = data['server']
        self.key = data['key']
        if new_ts:
            self.ts = data['ts']

    async def __aenter__(self) -> 'LP':
        if self._session is None:
            self._session = ClientSession()
            self.__session_owner = True
        return self

    async def __aexit__(self, *_) -> None:
        if self.__session_owner:
            await self._session.close()

    async def listen(self) -> AsyncGenerator[list, None]:
        """
        Асинхронный генератор результатов метода check()
        """
        while True:
            for update in await self.check():
                yield update



import json

from typing import Optional, Tuple, Union
from aiohttp import ClientSession
from warnings import warn

from vkmini.exceptions import NetworkError


default_session: Union[ClientSession, None] = None
json_decoder = json.loads


def set_decoder(loads) -> None:
    """
    Устанавливает функцию, которая будет использоваться при
    десериализации ответа на запросы
    """
    global json_decoder
    json_decoder = loads


def set_session(session: ClientSession) -> None:
    """
    Устанавливает сессию, которая будет использоваться по умолчанию при
    любых запросах к API, в т.ч. LongPoll (исключение - обращение к API c
    помощью обектов, при создании которых была явно передана сессия)

    Разумеется, стоит позаботиться о закрытии этой сессии перед завершением
    работы приложения

    Если при запросе не указана сессия (не установлена стандартная или не
    передана в конструктор при создании класса), на каждый новый запрос
    будет создаваться новая сессия (это плохое решение, лучше создать и
    указать сессию)
    """
    global default_session
    default_session = session


async def _get_session(session) -> Tuple[ClientSession, bool]:
    if not (default_session or session):
        return ClientSession(), True
    return (session or default_session), False


def _check_longpoll_session(session: Optional[ClientSession]):
    if not (session or default_session):
        warn(ResourceWarning(
            'При создании экземпляра LP не была указана сессия, при этом '
            'сессия default_session также не задана. Это приведёт к '
            'созданию новой сессии на каждый запрос.'
        ))


async def post(url: str,
               data: dict,
               client_session: ClientSession = None) -> dict:
    session, should_close = await _get_session(client_session)
    async with session.post(url, data=data) as resp:
        if resp.status == 200:
            data = await resp.json(loads=json_decoder)
        else:
            data = {}
    if should_close:
        await session.close()
    if data:
        return data
    raise NetworkError(resp.status)


async def longpoll_get(url: str,
                       client_session: ClientSession = None) -> dict:
    session, should_close = await _get_session(client_session)
    async with session.get(url) as resp:
        if resp.status == 200:
            data = await resp.json(loads=json_decoder)
        else:
            data = {}
    if should_close:
        await session.close()
    if data:
        return data
    raise NetworkError(resp.status)


async def get(url: str, client_session: ClientSession = None) -> bytes:
    session, should_close = await _get_session(client_session)
    try:
        async with session.get(url) as resp:
            if resp.status == 200:
                return await resp.content.read()
            else:
                raise NetworkError(resp.status)
    finally:
        if should_close:
            await session.close()



from typing import TYPE_CHECKING, Any, Dict, List, Union, Literal, Optional
from typing_extensions import TypedDict, NotRequired

if TYPE_CHECKING:
    from . import objects, methods, responses


class account_changePassword_response(TypedDict):
    token: str  # New token
    secret: NotRequired[str]  # New secret

class account_getActiveOffers_response(TypedDict):
    count: int  # Total number
    items: List['objects.account_offer']

account_getAppPermissions_response: int  # Permissions mask

class account_getBanned_response(TypedDict):
    count: int  # Total number
    items: List[int]
    profiles: NotRequired[List['objects.users_user_min']]
    groups: NotRequired[List['objects.groups_group']]

account_getCounters_response: 'objects.account_account_counters'

account_getInfo_response: 'objects.account_info'

account_getProfileInfo_response: 'objects.account_user_settings'

account_getPushSettings_response: 'objects.account_push_settings'

class account_saveProfileInfo_response(TypedDict):
    changed: NotRequired['objects.base_bool_int']  # 1 if changes has been processed
    name_request: NotRequired['objects.account_name_request']

ads_addOfficeUsers_response: bool  # true if success

ads_checkLink_response: 'objects.ads_link_status'

ads_createAds_response: List[int]

ads_createCampaigns_response: List[int]

ads_createClients_response: List[int]

class ads_createTargetGroup_response(TypedDict):
    id: NotRequired[int]  # Group ID
    pixel: NotRequired[str]  # Pixel code

ads_deleteAds_response: List[int]

ads_deleteCampaigns_response: int  # 0 if success

ads_deleteClients_response: int  # 0 if sucess

ads_getAccounts_response: List['objects.ads_account']

ads_getAdsLayout_response: List['objects.ads_ad_layout']

ads_getAdsTargeting_response: List['objects.ads_targ_settings']

ads_getAds_response: List['objects.ads_ad']

ads_getBudget_response: int  # Budget

ads_getCampaigns_response: List['objects.ads_campaign']

class ads_getCategories_response(TypedDict):
    v1: NotRequired[List['objects.ads_category']]  # Old categories
    v2: NotRequired[List['objects.ads_category']]  # Actual categories

ads_getClients_response: List['objects.ads_client']

ads_getDemographics_response: List['objects.ads_demo_stats']

ads_getFloodStats_response: 'objects.ads_flood_stats'

class ads_getLookalikeRequests_response(TypedDict):
    count: int  # Total count of found lookalike requests
    items: List['objects.ads_lookalike_request']  # found lookalike requests

class ads_getMusicians_response(TypedDict):
    items: List['objects.ads_musician']  # Musicians

ads_getOfficeUsers_response: List['objects.ads_users']

ads_getPostsReach_response: List['objects.ads_promoted_post_reach']

ads_getRejectionReason_response: 'objects.ads_reject_reason'

ads_getStatistics_response: List['objects.ads_stats']

ads_getSuggestions_cities_response: List['objects.ads_targ_suggestions_cities']

ads_getSuggestions_regions_response: List['objects.ads_targ_suggestions_regions']

ads_getSuggestions_response: List['objects.ads_targ_suggestions']

ads_getSuggestions_schools_response: List['objects.ads_targ_suggestions_schools']

ads_getTargetGroups_response: List['objects.ads_target_group']

ads_getTargetingStats_response: 'objects.ads_targ_stats'

ads_getUploadURL_response: str  # Photo upload URL

ads_getVideoUploadURL_response: str  # Video upload URL

ads_importTargetContacts_response: int  # Imported contacts number

ads_removeOfficeUsers_response: bool  # true if success

ads_updateAds_response: List[int]

ads_updateCampaigns_response: int  # Campaign ID

ads_updateClients_response: int  # Client ID

ads_updateOfficeUsers_response: List['objects.ads_updateOfficeUsers_result']

class adsweb_getAdCategories_response(TypedDict):
    categories: List['objects.adsweb_getAdCategories_response_categories_category']

class adsweb_getAdUnitCode_response(TypedDict):
    html: str

class adsweb_getAdUnits_response(TypedDict):
    count: int
    ad_units: NotRequired[List['objects.adsweb_getAdUnits_response_ad_units_ad_unit']]

class adsweb_getFraudHistory_response(TypedDict):
    count: int
    entries: NotRequired[List['objects.adsweb_getFraudHistory_response_entries_entry']]

class adsweb_getSites_response(TypedDict):
    count: int
    sites: NotRequired[List['objects.adsweb_getSites_response_sites_site']]

class adsweb_getStatistics_response(TypedDict):
    next_page_id: NotRequired[str]
    items: List['objects.adsweb_getStatistics_response_items_item']

class appWidgets_getAppImageUploadServer_response(TypedDict):
    upload_url: NotRequired[str]  # To upload an image, generate POST-request to upload_url with a file in photo field. Then call appWidgets.saveAppImage method

appWidgets_getAppImages_response: 'objects.appWidgets_photos'

class appWidgets_getGroupImageUploadServer_response(TypedDict):
    upload_url: NotRequired[str]  # To upload an image, generate POST-request to upload_url with a file in photo field. Then call appWidgets.saveAppImage method

appWidgets_getGroupImages_response: 'objects.appWidgets_photos'

appWidgets_getImagesById_response: List['objects.appWidgets_photo']

appWidgets_saveAppImage_response: 'objects.appWidgets_photo'

appWidgets_saveGroupImage_response: 'objects.appWidgets_photo'

class apps_getCatalog_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.apps_app']]

class apps_getFriendsList_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.users_user_full']]

class apps_getLeaderboard_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.apps_leaderboard']]
    profiles: NotRequired[List['objects.users_user_min']]

class apps_getLeaderboard_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.apps_leaderboard']]

class apps_getScopes_response(TypedDict):
    count: int  # Total number
    items: List['objects.apps_scope']

apps_getScore_response: int  # Score number

class apps_get_response(TypedDict):
    count: NotRequired[int]  # Total number of applications
    items: NotRequired[List['objects.apps_app']]  # List of applications

apps_sendRequest_response: int  # Request ID

class auth_restore_response(TypedDict):
    success: NotRequired[Literal[1]]  # 1 if success
    sid: NotRequired[str]  # Parameter needed to grant access by code

base_bool_response: 'objects.base_bool_int'

base_getUploadServer_response: 'objects.base_upload_server'

base_ok_response: Literal[1]

board_addTopic_response: int  # Topic ID

board_createComment_response: int  # Comment ID

class board_getComments_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.board_topic_comment']
    poll: NotRequired['objects.board_topic_poll']
    profiles: List['objects.users_user']
    groups: List['objects.groups_group']

class board_getComments_response(TypedDict):
    count: int  # Total number
    items: List['objects.board_topic_comment']
    poll: NotRequired['objects.board_topic_poll']

class board_getTopics_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.board_topic']
    default_order: 'objects.board_default_order'
    can_add_topics: 'objects.base_bool_int'  # Information whether current user can add topic
    profiles: List['objects.users_user_min']

class board_getTopics_response(TypedDict):
    count: int  # Total number
    items: List['objects.board_topic']
    default_order: 'objects.board_default_order'
    can_add_topics: 'objects.base_bool_int'  # Information whether current user can add topic

class database_getChairs_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.base_object']]

database_getCitiesById_response: List['objects.base_object']

class database_getCities_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_city']]

database_getCountriesById_response: List['objects.base_country']

class database_getCountries_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.base_country']]

class database_getFaculties_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_faculty']]

database_getMetroStationsById_response: List['objects.database_station']

class database_getMetroStations_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_station']]

class database_getRegions_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_region']]

database_getSchoolClasses_response: Union[str, int]

class database_getSchools_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_school']]

class database_getUniversities_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.database_university']]

class docs_add_response(TypedDict):
    id: NotRequired[int]  # Doc ID

docs_getById_response: List['objects.docs_doc']

class docs_getTypes_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.docs_doc_types']]

docs_getUploadServer: 'objects.base_upload_server'

class docs_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.docs_doc']

class docs_save_response(TypedDict):
    type: NotRequired['objects.docs_doc_attachment_type']
    audio_message: NotRequired['objects.messages_audio_message']
    doc: NotRequired['objects.docs_doc']
    graffiti: NotRequired['objects.messages_graffiti']

class docs_search_response(TypedDict):
    count: int  # Total number
    items: List['objects.docs_doc']

class downloadedGames_paid_status_response(TypedDict):
    is_paid: bool  # Game has been paid

fave_addTag_response: 'objects.fave_tag'

class fave_getPages_response(TypedDict):
    count: NotRequired[int]
    items: NotRequired[List['objects.fave_page']]

class fave_getTags_response(TypedDict):
    count: NotRequired[int]
    items: NotRequired[List['objects.fave_tag']]

class fave_get_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.fave_bookmark']]
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group']]

class fave_get_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.fave_bookmark']]

class friends_addList_response(TypedDict):
    list_id: int  # List ID

friends_add_response: Literal[1, 2, 4]  # Friend request status

friends_areFriends_extended_response: List['objects.friends_friend_extended_status']

friends_areFriends_response: List['objects.friends_friend_status']

class friends_delete_response(TypedDict):
    success: int
    friend_deleted: NotRequired[Literal[1]]  # Returns 1 if friend has been deleted
    out_request_deleted: NotRequired[Literal[1]]  # Returns 1 if out request has been canceled
    in_request_deleted: NotRequired[Literal[1]]  # Returns 1 if incoming request has been declined
    suggestion_deleted: NotRequired[Literal[1]]  # Returns 1 if suggestion has been declined

friends_getAppUsers_response: List[int]

friends_getByPhones_response: List['objects.friends_user_xtr_phone']

class friends_getLists_response(TypedDict):
    count: int  # Total number of friends lists
    items: List['objects.friends_friends_list']

friends_getMutual_response: List[int]

friends_getMutual_target_uids_response: List['objects.friends_mutual_friend']

class friends_getOnline_online_mobile_response(TypedDict):
    online: NotRequired[List[int]]
    online_mobile: NotRequired[List[int]]

friends_getOnline_response: List[int]

friends_getRecent_response: List[int]

class friends_getRequests_extended_response(TypedDict):
    count: NotRequired[int]  # Total requests number
    items: NotRequired[List['objects.friends_requests_xtr_message']]

class friends_getRequests_need_mutual_response(TypedDict):
    count: NotRequired[int]  # Total requests number
    items: NotRequired[List['objects.friends_requests']]

class friends_getRequests_response(TypedDict):
    count: NotRequired[int]  # Total requests number
    items: NotRequired[List[int]]
    count_unread: NotRequired[int]  # Total unread requests number

class friends_getSuggestions_response(TypedDict):
    count: int  # Total results number
    items: List['objects.users_user_full']

class friends_get_fields_response(TypedDict):
    count: int  # Total friends number
    items: List['objects.friends_user_xtr_lists']

class friends_get_response(TypedDict):
    count: int  # Total friends number
    items: List[int]

class friends_search_response(TypedDict):
    count: int  # Total number
    items: List['objects.users_user_full']

class gifts_get_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.gifts_gift']]

groups_addAddress_response: 'objects.groups_address'

class groups_addCallbackServer_response(TypedDict):
    server_id: NotRequired[int]

groups_addLink_response: 'objects.groups_group_link'

groups_create_response: 'objects.groups_group'

groups_editAddress_response: 'objects.groups_address'  # Result

class groups_getAddresses_response(TypedDict):
    count: int  # Total count of addresses
    items: List['objects.groups_address']

class groups_getBanned_response(TypedDict):
    count: int  # Total users number
    items: List['objects.groups_banned_item']

groups_getById_response: List['objects.groups_group_full']

class groups_getCallbackConfirmationCode_response(TypedDict):
    code: NotRequired[str]  # Confirmation code

class groups_getCallbackServers_response(TypedDict):
    count: int
    items: List['objects.groups_callback_server']

groups_getCallbackSettings_response: 'objects.groups_callback_settings'

class groups_getCatalogInfo_extended_response(TypedDict):
    enabled: int  # Information whether catalog is enabled for current user
    categories: NotRequired[List['objects.groups_group_category_full']]

class groups_getCatalogInfo_response(TypedDict):
    enabled: int  # Information whether catalog is enabled for current user
    categories: NotRequired[List['objects.groups_group_category']]

class groups_getCatalog_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.groups_group']

class groups_getInvitedUsers_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.users_user_full']

class groups_getInvites_extended_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.groups_group_xtr_invited_by']
    profiles: List['objects.users_user_min']
    groups: List['objects.groups_group_full']

class groups_getInvites_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.groups_group_xtr_invited_by']

groups_getLongPollServer_response: 'objects.groups_long_poll_server'

groups_getLongPollSettings_response: 'objects.groups_long_poll_settings'

class groups_getMembers_fields_response(TypedDict):
    count: int  # Total members number
    items: List['objects.groups_user_xtr_role']

class groups_getMembers_filter_response(TypedDict):
    count: int  # Total members number
    items: List['objects.groups_member_role']

class groups_getMembers_response(TypedDict):
    count: int  # Total members number
    items: List[int]

class groups_getRequests_fields_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.users_user_full']

class groups_getRequests_response(TypedDict):
    count: int  # Total communities number
    items: List[int]

class groups_getSettings_response(TypedDict):
    access: NotRequired['objects.groups_group_access']  # Community access settings
    address: NotRequired[str]  # Community's page domain
    audio: 'objects.groups_group_audio'  # Audio settings
    articles: int  # Articles settings
    city_id: int  # City id of group
    contacts: NotRequired['objects.base_bool_int']
    links: NotRequired['objects.base_bool_int']
    sections_list: NotRequired[Dict[str, Any]]
    main_section: NotRequired['objects.groups_group_full_main_section']
    secondary_section: NotRequired[int]
    age_limits: NotRequired[int]
    country_id: int  # Country id of group
    description: str  # Community description
    docs: 'objects.groups_group_docs'  # Docs settings
    events: NotRequired['objects.base_bool_int']
    obscene_filter: 'objects.base_bool_int'  # Information whether the obscene filter is enabled
    obscene_stopwords: 'objects.base_bool_int'  # Information whether the stopwords filter is enabled
    obscene_words: List[str]  # The list of stop words
    event_group_id: NotRequired[int]
    photos: int  # Photos settings
    public_category: NotRequired[int]  # Information about the group category
    public_category_list: NotRequired[List['objects.groups_group_public_category_list']]
    public_date: NotRequired[str]
    public_date_label: NotRequired[str]
    public_subcategory: NotRequired[int]  # Information about the group subcategory
    rss: NotRequired[str]  # URL of the RSS feed
    start_date: NotRequired[int]  # Start date
    finish_date: NotRequired[int]  # Finish date in Unixtime format
    subject: NotRequired[int]  # Community subject ID
    subject_list: NotRequired[List['objects.groups_subject_item']]
    suggested_privacy: NotRequired[int]
    title: str  # Community title
    topics: 'objects.groups_group_topics'  # Topics settings
    twitter: NotRequired['objects.groups_settings_twitter']
    video: 'objects.groups_group_video'  # Video settings
    wall: 'objects.groups_group_wall'  # Wall settings
    website: NotRequired[str]  # Community website
    phone: NotRequired[str]  # Community phone
    email: NotRequired[str]  # Community email
    wiki: 'objects.groups_group_wiki'  # Wiki settings

groups_getTagList_response: List['objects.groups_group_tag']

class groups_getTokenPermissions_response(TypedDict):
    mask: int
    permissions: List['objects.groups_token_permission_setting']

class groups_get_extended_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.groups_group_full']

class groups_get_response(TypedDict):
    count: int  # Total communities number
    items: List[int]

class groups_isMember_extended_response(TypedDict):
    member: 'objects.base_bool_int'  # Information whether user is a member of the group
    invitation: NotRequired['objects.base_bool_int']  # Information whether user has been invited to the group
    can_invite: NotRequired['objects.base_bool_int']  # Information whether user can be invited
    can_recall: NotRequired['objects.base_bool_int']  # Information whether user's invite to the group can be recalled
    request: NotRequired['objects.base_bool_int']  # Information whether user has sent request to the group

groups_isMember_response: 'objects.base_bool_int'  # Information whether user is a member of the group

groups_isMember_user_ids_extended_response: List['objects.groups_member_status_full']

groups_isMember_user_ids_response: List['objects.groups_member_status']

class groups_search_response(TypedDict):
    count: int  # Total communities number
    items: List['objects.groups_group']

leads_checkUser_response: 'objects.leads_checked'

leads_complete_response: 'objects.leads_complete'

leads_getStats_response: 'objects.leads_lead'

leads_getUsers_response: List['objects.leads_entry']

class leads_metricHit_response(TypedDict):
    result: NotRequired[bool]  # Information whether request has been processed successfully
    redirect_link: NotRequired[str]  # Redirect link

leads_start_response: 'objects.leads_start'

class likes_add_response(TypedDict):
    likes: int  # Total likes number

class likes_delete_response(TypedDict):
    likes: int  # Total likes number

class likes_getList_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.users_user_min']

class likes_getList_response(TypedDict):
    count: int  # Total number
    items: List[int]

class likes_isLiked_response(TypedDict):
    liked: 'objects.base_bool_int'  # Information whether user liked the object
    copied: 'objects.base_bool_int'  # Information whether user reposted the object

class market_addAlbum_response(TypedDict):
    market_album_id: NotRequired[int]  # Album ID

class market_add_response(TypedDict):
    market_item_id: NotRequired[int]  # Item ID

market_createComment_response: int  # Comment ID

market_deleteComment_response: 'objects.base_bool_int'  # Returns 1 if request has been processed successfully (0 if the comment is not found)

class market_getAlbumById_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_album']]

class market_getAlbums_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_album']]

class market_getById_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item_full']]

class market_getById_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item']]

class market_getCategories_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_category']]

class market_getComments_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.wall_wall_comment']]

class market_getGroupOrders_response(TypedDict):
    count: int  # Total number
    items: List['objects.market_order']

class market_getOrderById_response(TypedDict):
    order: NotRequired['objects.market_order']

class market_getOrderItems_response(TypedDict):
    count: int  # Total number
    items: List['objects.market_order_item']

class market_getOrders_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.market_order']
    groups: NotRequired[List['objects.groups_group_full']]

class market_getOrders_response(TypedDict):
    count: int  # Total number
    items: List['objects.market_order']

class market_get_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item_full']]

class market_get_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item']]

market_restoreComment_response: 'objects.base_bool_int'  # Returns 1 if request has been processed successfully (0 if the comment is not found)

class market_search_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item_full']]

class market_search_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.market_market_item']]

messages_createChat_response: int  # Chat ID

class messages_deleteChatPhoto_response(TypedDict):
    message_id: NotRequired[int]  # Service message ID
    chat: NotRequired['objects.messages_chat']

class messages_deleteConversation_response(TypedDict):
    last_deleted_id: int  # Id of the last message, that was deleted

messages_delete_response: Dict[str, Any]

messages_edit_response: 'objects.base_bool_int'  # Result

class messages_getByConversationMessageId_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_message']

class messages_getById_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_message']
    profiles: List['objects.users_user_full']
    groups: NotRequired[List['objects.groups_group_full']]

class messages_getById_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_message']

class messages_getChatPreview_response(TypedDict):
    preview: NotRequired['objects.messages_chat_preview']
    profiles: NotRequired[List['objects.users_user_full']]

messages_getChat_chat_ids_fields_response: List['objects.messages_chat_full']

messages_getChat_chat_ids_response: List['objects.messages_chat']

messages_getChat_fields_response: 'objects.messages_chat_full'

messages_getChat_response: 'objects.messages_chat'

class messages_getConversationMembers_response(TypedDict):
    count: int  # Chat members count
    items: List['objects.messages_conversation_member']
    chat_restrictions: NotRequired['objects.messages_chat_restrictions']
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]

class messages_getConversationsById_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_conversation']
    profiles: NotRequired[List['objects.users_user']]

class messages_getConversationsById_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_conversation']

class messages_getConversations_response(TypedDict):
    count: int  # Total number
    unread_count: NotRequired[int]  # Unread dialogs number
    items: List['objects.messages_conversation_with_message']
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]

class messages_getHistoryAttachments_response(TypedDict):
    items: NotRequired[List['objects.messages_history_attachment']]
    next_from: NotRequired[str]  # Value for pagination

class messages_getHistory_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_message']
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]

class messages_getImportantMessages_extended_response(TypedDict):
    messages: 'objects.messages_messages_array'
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List['objects.groups_group']]
    conversations: NotRequired[List['objects.messages_conversation']]

class messages_getImportantMessages_response(TypedDict):
    messages: 'objects.messages_messages_array'
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List['objects.groups_group']]
    conversations: NotRequired[List['objects.messages_conversation']]

class messages_getInviteLink_response(TypedDict):
    link: NotRequired[str]

messages_getLastActivity_response: 'objects.messages_last_activity'

class messages_getLongPollHistory_response(TypedDict):
    history: NotRequired[int]
    groups: NotRequired[List['objects.groups_group']]
    messages: NotRequired['objects.messages_longpoll_messages']
    profiles: NotRequired[List['objects.users_user_full']]
    chats: NotRequired[List['objects.messages_chat']]
    new_pts: NotRequired[int]  # Persistence timestamp
    more: NotRequired[bool]  # Has more
    conversations: NotRequired[List['objects.messages_conversation']]

messages_getLongPollServer_response: 'objects.messages_longpoll_params'

class messages_isMessagesFromGroupAllowed_response(TypedDict):
    is_allowed: NotRequired['objects.base_bool_int']

class messages_joinChatByInviteLink_response(TypedDict):
    chat_id: NotRequired[int]

messages_markAsImportant_response: List[int]

messages_pin_response: 'objects.messages_pinned_message'

class messages_searchConversations_response(TypedDict):
    count: NotRequired[int]  # Total results number
    items: NotRequired[List['objects.messages_conversation']]
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]

class messages_search_response(TypedDict):
    count: int  # Total number
    items: List['objects.messages_message']

messages_send_response: int  # Message ID

messages_send_user_ids_response: Dict[str, Any]

class messages_setChatPhoto_response(TypedDict):
    message_id: NotRequired[int]  # Service message ID
    chat: NotRequired['objects.messages_chat']

class newsfeed_getBanned_extended_response(TypedDict):
    groups: NotRequired[List['objects.users_user_full']]
    profiles: NotRequired[List['objects.groups_group_full']]

class newsfeed_getBanned_response(TypedDict):
    groups: NotRequired[List[int]]
    members: NotRequired[List[int]]

class newsfeed_getComments_response(TypedDict):
    items: List['objects.newsfeed_newsfeed_item']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']
    next_from: NotRequired[str]  # New from value

class newsfeed_getLists_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.newsfeed_list_full']

class newsfeed_getLists_response(TypedDict):
    count: int  # Total number
    items: List['objects.newsfeed_list']

class newsfeed_getMentions_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wallpost_to_id']

class newsfeed_getRecommended_response(TypedDict):
    items: NotRequired[List['objects.newsfeed_newsfeed_item']]
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]
    new_offset: NotRequired[str]  # New offset value
    next_from: NotRequired[str]  # Next from value

class newsfeed_getSuggestedSources_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[Union['objects.groups_group_full', 'objects.users_user_xtr_type']]

class newsfeed_get_response(TypedDict):
    items: NotRequired[List['objects.newsfeed_newsfeed_item']]
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]
    next_from: NotRequired[str]  # New from value

newsfeed_saveList_response: int  # List ID

class newsfeed_search_extended_response(TypedDict):
    items: NotRequired[List['objects.wall_wallpost_full']]
    profiles: NotRequired[List['objects.users_user_full']]
    groups: NotRequired[List['objects.groups_group_full']]
    suggested_queries: NotRequired[List[str]]
    next_from: NotRequired[str]
    count: NotRequired[int]  # Filtered number
    total_count: NotRequired[int]  # Total number

class newsfeed_search_response(TypedDict):
    items: NotRequired[List['objects.wall_wallpost_full']]
    suggested_queries: NotRequired[List[str]]
    next_from: NotRequired[str]
    count: NotRequired[int]  # Filtered number
    total_count: NotRequired[int]  # Total number

notes_add_response: int  # Note ID

notes_createComment_response: int  # Comment ID

notes_getById_response: 'objects.notes_note'

class notes_getComments_response(TypedDict):
    count: int  # Total number
    items: List['objects.notes_note_comment']

class notes_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.notes_note']

class notifications_get_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.notifications_notification_item']]
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List['objects.groups_group']]
    last_viewed: NotRequired[int]  # Time when user has been checked notifications last time
    photos: NotRequired[List['objects.photos_photo']]
    videos: NotRequired[List['objects.video_video']]
    apps: NotRequired[List['objects.apps_app']]
    next_from: NotRequired[str]
    ttl: NotRequired[int]

notifications_markAsViewed_response: 'objects.base_bool_int'  # Result

notifications_sendMessage_response: List['objects.notifications_send_message_item']

orders_cancelSubscription_response: 'objects.base_bool_int'  # Result

orders_changeState_response: str  # New state

orders_getAmount_response: 'objects.orders_amount'

orders_getById_response: List['objects.orders_order']

orders_getUserSubscriptionById_response: 'objects.orders_subscription'

class orders_getUserSubscriptions_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.orders_subscription']]

orders_get_response: List['objects.orders_order']

orders_updateSubscription_response: 'objects.base_bool_int'  # Result

pages_getHistory_response: List['objects.pages_wikipage_history']

pages_getTitles_response: List['objects.pages_wikipage']

pages_getVersion_response: 'objects.pages_wikipage_full'

pages_get_response: 'objects.pages_wikipage_full'

pages_parseWiki_response: str  # HTML source

pages_saveAccess_response: int  # Page ID

pages_save_response: int  # Page ID

photos_copy_response: int  # Photo ID

photos_createAlbum_response: 'objects.photos_photo_album_full'

photos_createComment_response: int  # Created comment ID

photos_deleteComment_response: 'objects.base_bool_int'  # Returns 1 if request has been processed successfully, 0 if the comment is not found

photos_getAlbumsCount_response: int  # Albums number

class photos_getAlbums_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo_album_full']

class photos_getAllComments_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.photos_comment_xtr_pid']]

class photos_getAll_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.photos_photo_full_xtr_real_offset']]
    more: NotRequired['objects.base_bool_int']  # Information whether next page is presented

class photos_getAll_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.photos_photo_xtr_real_offset']]
    more: NotRequired['objects.base_bool_int']  # Information whether next page is presented

photos_getById_extended_response: List['objects.photos_photo_full']

photos_getById_response: List['objects.photos_photo']

class photos_getComments_extended_response(TypedDict):
    count: int  # Total number
    real_offset: NotRequired[int]  # Real offset of the comments
    items: List['objects.wall_wall_comment']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

class photos_getComments_response(TypedDict):
    count: NotRequired[int]  # Total number
    real_offset: NotRequired[int]  # Real offset of the comments
    items: NotRequired[List['objects.wall_wall_comment']]

photos_getMarketUploadServer_response: 'objects.base_upload_server'

photos_getMessagesUploadServer_response: 'objects.photos_photo_upload'

class photos_getNewTags_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo_xtr_tag_info']

photos_getTags_response: List['objects.photos_photo_tag']

photos_getUploadServer_response: 'objects.photos_photo_upload'

class photos_getUserPhotos_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo_full']

class photos_getUserPhotos_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo']

photos_getWallUploadServer_response: 'objects.photos_photo_upload'

class photos_get_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo_full']

class photos_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.photos_photo']

photos_putTag_response: int  # Created tag ID

photos_restoreComment_response: 'objects.base_bool_int'  # Returns 1 if request has been processed successfully, 0 if the comment is not found

photos_saveMarketAlbumPhoto_response: List['objects.photos_photo']

photos_saveMarketPhoto_response: List['objects.photos_photo']

photos_saveMessagesPhoto_response: List['objects.photos_photo']

photos_saveOwnerCoverPhoto_response: List['objects.base_image']

class photos_saveOwnerPhoto_response(TypedDict):
    photo_hash: str  # Photo hash
    photo_src: str  # Uploaded image url
    photo_src_big: NotRequired[str]  # Uploaded image url
    photo_src_small: NotRequired[str]  # Uploaded image url
    saved: NotRequired[int]  # Returns 1 if profile photo is saved
    post_id: NotRequired[int]  # Created post ID

photos_saveWallPhoto_response: List['objects.photos_photo']

photos_save_response: List['objects.photos_photo']

class photos_search_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.photos_photo']]

podcasts_getPopular_response: List['objects.podcast_popular_podcast']

podcasts_getRecentSearchRequests_response: List[str]

class podcasts_search_response(TypedDict):
    podcasts: NotRequired[List['objects.podcast_podcast']]
    episodes: NotRequired[List['objects.audio_audio']]
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List[Any]]

polls_addVote_response: 'objects.base_bool_int'  # Result

polls_create_response: 'objects.polls_poll'

polls_deleteVote_response: 'objects.base_bool_int'  # Result

polls_getById_response: 'objects.polls_poll'

polls_getVoters_response: List['objects.polls_voters']

class prettyCards_create_response(TypedDict):
    owner_id: int  # Owner ID of created pretty card
    card_id: str  # Card ID of created pretty card

class prettyCards_delete_response(TypedDict):
    owner_id: int  # Owner ID of deleted pretty card
    card_id: str  # Card ID of deleted pretty card
    error: NotRequired[str]  # Error reason if error happened

class prettyCards_edit_response(TypedDict):
    owner_id: int  # Owner ID of edited pretty card
    card_id: str  # Card ID of edited pretty card

prettyCards_getById_response: List['objects.prettyCards_prettyCard']

prettyCards_getUploadURL_response: str  # Upload URL

class prettyCards_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.prettyCards_prettyCard']

class search_getHints_response(TypedDict):
    count: int
    items: List['objects.search_hint']
    suggested_queries: NotRequired[List[str]]

secure_checkToken_response: 'objects.secure_token_checked'

secure_getAppBalance_response: int  # App balance

secure_getSMSHistory_response: List['objects.secure_sms_notification']

secure_getTransactionsHistory_response: List['objects.secure_transaction']

secure_getUserLevel_response: List['objects.secure_level']

secure_giveEventSticker_response: Dict[str, Any]

secure_sendNotification_response: List[int]

stats_getPostReach_response: List['objects.stats_wallpost_stat']

stats_get_response: List['objects.stats_period']

status_get_response: 'objects.status_status'

storage_getKeys_response: List[str]

storage_get_response: str  # Value of key

storage_get_v5110_response: List['objects.storage_value']

storage_get_with_keys_response: List['objects.storage_value']

class stories_getBanned_extended_response(TypedDict):
    count: int  # Stories count
    items: List[int]
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

class stories_getBanned_response(TypedDict):
    count: int  # Stories count
    items: List[int]

class stories_getById_extended_response(TypedDict):
    count: int  # Stories count
    items: List['objects.stories_story']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

class stories_getById_response(TypedDict):
    count: int  # Stories count
    items: List['objects.stories_story']

class stories_getPhotoUploadServer_response(TypedDict):
    upload_url: str  # Upload URL
    user_ids: List[int]  # Users ID who can to see story.

stories_getStats_response: 'objects.stories_story_stats'

class stories_getVideoUploadServer_response(TypedDict):
    upload_url: str  # Upload URL
    user_ids: List[int]  # Users ID who can to see story.

class stories_getViewers_extended_V5115_response(TypedDict):
    count: int  # Viewers count
    items: List['objects.stories_viewers_item']
    hidden_reason: NotRequired[str]

class stories_getViewers_extended_response(TypedDict):
    count: int  # Viewers count
    items: List['objects.users_user_full']

class stories_get_V5113_response(TypedDict):
    count: int
    items: List['objects.stories_feed_item']
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List['objects.groups_group']]
    need_upload_screen: NotRequired[bool]

class stories_get_response(TypedDict):
    count: int  # Stories count
    items: 'objects.stories_story'
    promo_data: NotRequired['objects.stories_promo_block']
    profiles: NotRequired[List['objects.users_user']]
    groups: NotRequired[List['objects.groups_group']]
    need_upload_screen: NotRequired[bool]

class stories_save_response(TypedDict):
    count: int
    items: List['objects.stories_story']

class stories_upload_response(TypedDict):
    upload_result: NotRequired[str]  # A string hash that is used in the stories.save method

class streaming_getServerUrl_response(TypedDict):
    endpoint: NotRequired[str]  # Server host
    key: NotRequired[str]  # Access key

class users_getFollowers_fields_response(TypedDict):
    count: int  # Total number of available results
    items: List['objects.users_user_full']

class users_getFollowers_response(TypedDict):
    count: int  # Total friends number
    items: List[int]

class users_getSubscriptions_extended_response(TypedDict):
    count: int  # Total number of available results
    items: List['objects.users_subscriptions_item']

class users_getSubscriptions_response(TypedDict):
    users: 'objects.users_users_array'
    groups: 'objects.groups_groups_array'

users_get_response: List['objects.users_user_xtr_counters']

class users_search_response(TypedDict):
    count: NotRequired[int]  # Total number of available results
    items: NotRequired[List['objects.users_user_full']]

utils_checkLink_response: 'objects.utils_link_checked'

class utils_getLastShortenedLinks_response(TypedDict):
    count: NotRequired[int]  # Total number of available results
    items: NotRequired[List['objects.utils_last_shortened_link']]

utils_getLinkStats_extended_response: 'objects.utils_link_stats_extended'

utils_getLinkStats_response: 'objects.utils_link_stats'

utils_getServerTime_response: int  # Time as Unixtime

utils_getShortLink_response: 'objects.utils_short_link'

utils_resolveScreenName_response: 'objects.utils_domain_resolved'

class video_addAlbum_response(TypedDict):
    album_id: int  # Created album ID

video_createComment_response: int  # Created comment ID

video_getAlbumById_response: 'objects.video_video_album_full'

class video_getAlbumsByVideo_extended_response(TypedDict):
    count: NotRequired[int]  # Total number
    items: NotRequired[List['objects.video_video_album_full']]

video_getAlbumsByVideo_response: List[int]

class video_getAlbums_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video_album_full']

class video_getAlbums_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video_album_full']

class video_getComments_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wall_comment']
    profiles: List['objects.users_user_min']
    groups: List['objects.groups_group_full']

class video_getComments_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wall_comment']

class video_get_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video_full']
    profiles: List['objects.users_user_min']
    groups: List['objects.groups_group_full']

class video_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video']

video_restoreComment_response: 'objects.base_bool_int'  # Returns 1 if request has been processed successfully, 0 if the comment is not found

video_save_response: 'objects.video_save_result'

class video_search_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video']
    profiles: List['objects.users_user_min']
    groups: List['objects.groups_group_full']

class video_search_response(TypedDict):
    count: int  # Total number
    items: List['objects.video_video']

class wall_createComment_response(TypedDict):
    comment_id: int  # Created comment ID

class wall_edit_response(TypedDict):
    post_id: int  # Edited post ID

class wall_getById_extended_response(TypedDict):
    items: List['objects.wall_wallpost_full']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

wall_getById_legacy_response: List['objects.wall_wallpost_full']

class wall_getById_response(TypedDict):
    items: NotRequired[List['objects.wall_wallpost_full']]

class wall_getComment_extended_response(TypedDict):
    items: List['objects.wall_wall_comment']
    profiles: List['objects.users_user']
    groups: List['objects.groups_group']

class wall_getComment_response(TypedDict):
    items: List['objects.wall_wall_comment']

class wall_getComments_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wall_comment']
    show_reply_button: NotRequired[bool]
    can_post: NotRequired[bool]  # Information whether current user can comment the post
    groups_can_post: NotRequired[bool]  # Information whether groups can comment the post
    current_level_count: NotRequired[int]  # Count of replies of current level
    profiles: List['objects.users_user']
    groups: List['objects.groups_group']

class wall_getComments_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wall_comment']
    can_post: NotRequired[bool]  # Information whether current user can comment the post
    groups_can_post: NotRequired[bool]  # Information whether groups can comment the post
    current_level_count: NotRequired[int]  # Count of replies of current level

class wall_getReposts_response(TypedDict):
    items: List['objects.wall_wallpost_full']
    profiles: List['objects.users_user']
    groups: List['objects.groups_group']

class wall_get_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wallpost_full']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

class wall_get_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wallpost_full']

class wall_postAdsStealth_response(TypedDict):
    post_id: int  # Created post ID

class wall_post_response(TypedDict):
    post_id: int  # Created post ID

class wall_repost_response(TypedDict):
    success: int
    post_id: int  # Created post ID
    reposts_count: int  # Reposts number
    wall_repost_count: int  # Reposts to wall number
    mail_repost_count: int  # Reposts to mail number
    likes_count: int  # Reposts number

class wall_search_extended_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wallpost_full']
    profiles: List['objects.users_user_full']
    groups: List['objects.groups_group_full']

class wall_search_response(TypedDict):
    count: int  # Total number
    items: List['objects.wall_wallpost_full']

class widgets_getComments_response(TypedDict):
    count: int  # Total number
    posts: List['objects.widgets_widget_comment']

class widgets_getPages_response(TypedDict):
    count: int  # Total number
    pages: List['objects.widgets_widget_page']




import time

from typing import Any, Callable, Protocol, Deque
from asyncio import Future
from collections import deque


class AbstractLogger(Protocol):
    def debug(self, __msg: str):
        raise NotImplementedError

    def warning(self, __msg: str):
        raise NotImplementedError


class Printer:
    """
    Реализует распространённый интерфейс логгера,
    выводит логируемые сообщения в sys.stdout
    """
    level: int

    def __init__(self, level: int = 2) -> None:
        self.level = level

    @staticmethod
    def __print(level_name, msg):
        print(
            time.strftime('%H:%M:%S', time.localtime()),
            level_name,
            msg
        )

    def critical(self, msg: Any):
        self.__print('CRITICAL', msg)

    def error(self, msg: Any):
        if self.level < 5:
            self.__print('ERROR', msg)

    def warning(self, msg: Any):
        if self.level < 4:
            self.__print('WARNING', msg)

    def info(self, msg: Any):
        if self.level < 3:
            self.__print('INFO', msg)

    def debug(self, msg: Any):
        if self.level < 2:
            self.__print('DEBUG', msg)


class Blocker:
    _waiters: Deque[Future]

    def __init__(self, future_factory: Callable[[], Future]):
        self._allowed = True
        self._waiters = deque()
        self._create_future = future_factory

    def is_allowed(self) -> bool:
        return self._allowed

    def release_one(self):
        while self._waiters:
            waiter = self._waiters.popleft()
            if not waiter.done():
                waiter.set_result(True)
                break
        if not self._waiters:
            self._allowed = True

    def block(self):
        self._allowed = False

    async def wait(self):
        if not self._allowed:
            fut = self._create_future()
            self._waiters.append(fut)
            await fut



from typing import Tuple


__all__: Tuple[str, ...] = (
    'VkApi',
    'GroupVkApi',
    'set_session',
    'set_decoder',
    'Keyboard',
    'Button',
    'VkResponseException',
    'LP',
    'GroupLP',
)


from vkmini.api import VkApi, GroupVkApi, set_loop
from vkmini.request import set_session, set_decoder
from vkmini.keyboard import Keyboard, Button
from vkmini.exceptions import VkResponseException
from vkmini.user_longpoll import LP
from vkmini.group_longpoll import GroupLP
